"""Exception module"""


class InvalidValueException(Exception):
    """Invalid value exception"""


class NoneValueException(Exception):
    """None value exception"""


class InvalidTypeException(Exception):
    """Invalid type exception"""
